 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>

/*

  This software is provided under the LGPL in January 2009 by the
  Cluster Science Centre
  QSAS team,
  Imperial College, London

  Copyright (C) 2009  Imperial College, London

  This is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Lesser Public License as published
  by the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

  This software is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  To received a copy of the GNU Library General Public License
  write to the Free Software Foundation, Inc., 
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

  History:
  
  January 2009:  v1.00
  Initial release.
  
  February 2009: v1.01
  Modify format in strfMJD() for "%." and "%{1-9}".
  
  February 2009: v1.02
  Modify format in getISOstring() from %g to %f.

  February 2009: v1.04
  Remove forceJulian flag.
  Add fixes from Alan Irwin of PLPLOT.
  
  March 2009: v1.05
  Improve formatting of seconds, round to required precision before calling breakDownMJD().
  
*/
	/* MJD measures from the start of 17 Nov 1858 */
	
	/* 
	 These utilities use the Gregorian calendar after 4 Oct 1582 (Julian) i.e. from 15 Oct 1582 Gregorian
	 Note C libraries use Gregorian only from 14 Sept 1752 
	 More detailed discussion can be found at http://aa.usno.navy.mil/data/docs/JulianDate.php
	 These routines have been compared with the results of the US Naval Observatory online converter.
	 Modified Julian Date (MJD) = Julian Date (JD) - 2400000.5
	 
	 In all routines, specifying a day, hour, minute or second field greater than would be valid is
	 handled with modulo arithmetic and safe. 
	 Thus 2006-12-32 00:62:00.0 will safely, and correctly, be treated as 2007-01-01 01:02:00.0 
	 
	*/

typedef struct MJDtimeStruct
{ 
	/* 
	   MJD starts at 0h, so truncating MJD always gives the same day whatever the time (unlike JD).
	   The MJD base day is arbitrary, i.e. seconds can be greater than one day or even negative.
	*/
	   
	int base_day; /* integer part of MJD used as default */
	double time_sec; /* seconds from start of base_day */
	
}MJDtime;


int setFromUT(int year, int month, int day, int hour, int min, double sec, MJDtime *MJD);
void setFromDOY(int year, int doy, int hour, int min, double sec, MJDtime *MJD);
void setFromBCE(int yearBCE, int month, int day, int hour, int min, double sec, MJDtime *MJD);
void setFromMJD(double ModifiedJulianDate, MJDtime *MJD);
void setFromCDFepoch(double cdfepoch, MJDtime *MJD);
void setFromJD(double JulianDate, MJDtime *MJD);
int setFromISOstring(const char* ISOstring, MJDtime *MJD);
int breakDownMJD(int *year, int *month, int *day, int *hour, int *min, double *sec, const MJDtime *MJD);
double getMJD(const MJDtime *MJD);
double getJD(const MJDtime *MJD);
double getDiffDays(MJDtime *MJD1, MJDtime *MJD2);
double getDiffSecs(MJDtime *MJD1, MJDtime *MJD2);
double getCDFepoch(MJDtime *MJD);
const char * getISOString(MJDtime *MJD, int delim);
const char * getDayOfWeek(const MJDtime *MJD);
const char * getLongDayOfWeek( const MJDtime *MJD);
const char * getMonth(int m);
const char * getLongMonth(int m);
void getYandD(int *year, int *isLeapyear, int *dofy, const MJDtime *MJD);
size_t strfMJD(char * buf, size_t len, const char *format, const MJDtime *MJD);
void setGregorianStartMJD(double GregorianMJD);
void normalize_MJD(MJDtime *MJDout, const MJDtime *MJDin);
char * getMJDtimeVersion();

